@@ -38,31 +38,28 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: MozDownload.cpp,v 1.31.4.8 2004/08/27 20:55:44 chpe Exp $
+ * $Id: MozDownload.cpp,v 1.47.4.2 2004/09/21 11:13:21 chpe Exp $
  */
 
+#include "mozilla-config.h"
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "MozDownload.h"
 #include "mozilla-download.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
 #include "ephy-file-helpers.h"
 #include "ephy-debug.h"
+#include "MozDownload.h"
+#include "EphyUtils.h"
 
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glib/gi18n.h>
 
-#include "nsIExternalHelperAppService.h"
-#include "nsDirectoryServiceDefs.h"
-#include "nsDirectoryServiceUtils.h"
-#include "nsIRequest.h"
-#include "nsIMIMEInfo.h"
-#include "nsIFileURL.h"
-#include "netCore.h"
-#include "nsNetUtil.h"
+#include <nsIFileURL.h>
+#include <nsEmbedString.h>
 
 const char* const persistContractID = "@mozilla.org/embedding/browser/nsWebBrowserPersist;1";
 
@@ -82,18 +79,10 @@ MozDownload::~MozDownload()
 	NS_ASSERTION (!mEphyDownload, "MozillaDownload still alive!");
 }
 
-#if MOZILLA_SNAPSHOT < 16
-NS_IMPL_ISUPPORTS2(MozDownload, nsIDownload, nsIWebProgressListener)
-#else
 NS_IMPL_ISUPPORTS3(MozDownload, nsIDownload, nsITransfer, nsIWebProgressListener)
-#endif
 
 NS_IMETHODIMP
-#if MOZILLA_SNAPSHOT < 16
-MozDownload::InitForEmbed (nsIURI *aSource, nsILocalFile *aTarget, const PRUnichar *aDisplayName,
-#else
 MozDownload::InitForEmbed (nsIURI *aSource, nsIURI *aTarget, const PRUnichar *aDisplayName,
-#endif
 		           nsIMIMEInfo *aMIMEInfo, PRInt64 startTime, nsIWebBrowserPersist *aPersist,
 		           MozillaEmbedPersist *aEmbedPersist, PRInt32 aMaxSize)
 {
@@ -103,15 +92,9 @@ MozDownload::InitForEmbed (nsIURI *aSource, nsIURI *aTarget, const PRUnichar *aD
 }
 
 /* void init (in nsIURI aSource, in nsIURI aTarget, in wstring aDisplayName, in nsIMIMEInfo aMIMEInfo, in long long startTime, in nsIWebBrowserPersist aPersist); */
-#if MOZILLA_SNAPSHOT < 16
-NS_IMETHODIMP
-MozDownload::Init(nsIURI *aSource, nsILocalFile *aTarget, const PRUnichar *aDisplayName,
-		  nsIMIMEInfo *aMIMEInfo, PRInt64 startTime, nsIWebBrowserPersist *aPersist)
-#else
 NS_IMETHODIMP
 MozDownload::Init(nsIURI *aSource, nsIURI *aTarget, const PRUnichar *aDisplayName,
 		  nsIMIMEInfo *aMIMEInfo, PRInt64 startTime, nsIWebBrowserPersist *aPersist)
-#endif
 {
 	PRBool addToView = PR_TRUE;
 
@@ -168,13 +151,8 @@ MozDownload::GetSource(nsIURI **aSource)
 	return NS_OK;
 }
 
-#if MOZILLA_SNAPSHOT < 16
-NS_IMETHODIMP
-MozDownload::GetTarget(nsILocalFile **aTarget)
-#else
 NS_IMETHODIMP
 MozDownload::GetTarget(nsIURI **aTarget)
-#endif
 {
 	NS_ENSURE_ARG_POINTER(aTarget);
 	NS_IF_ADDREF(*aTarget = mDestination);
@@ -182,7 +160,6 @@ MozDownload::GetTarget(nsIURI **aTarget)
 	return NS_OK;
 }
 
-#if MOZILLA_SNAPSHOT > 15
 NS_IMETHODIMP
 MozDownload::GetTargetFile (nsILocalFile** aTargetFile)
 {
@@ -197,7 +174,6 @@ MozDownload::GetTargetFile (nsILocalFile** aTargetFile)
 		rv = CallQueryInterface(file, aTargetFile);
 	return rv;
 }
-#endif
 
 NS_IMETHODIMP
 MozDownload::GetPersist(nsIWebBrowserPersist **aPersist)
@@ -277,16 +253,18 @@ MozDownload::GetElapsedTime(PRInt64 *aElapsedTime)
 NS_IMETHODIMP
 MozDownload::GetMIMEInfo(nsIMIMEInfo **aMIMEInfo)
 {
-	return NS_ERROR_NOT_IMPLEMENTED;
+        NS_ENSURE_ARG_POINTER(aMIMEInfo);
+	NS_IF_ADDREF(*aMIMEInfo = mMIMEInfo);
+
+	return NS_OK;
 }
 
 NS_IMETHODIMP
 MozDownload::GetListener(nsIWebProgressListener **aListener)
 {
 	NS_ENSURE_ARG_POINTER(aListener);
-	NS_IF_ADDREF(*aListener = (nsIWebProgressListener *)this);
 
-	return NS_OK;
+	return CallQueryInterface (this, aListener);
 }
 
 NS_IMETHODIMP
@@ -350,7 +328,24 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 		else if (NS_SUCCEEDED (aStatus))
 		{
 			GnomeVFSMimeApplication *helperApp;
-#if MOZILLA_SNAPSHOT < 20
+#ifdef MOZ_NSIMIMEINFO_NSACSTRING_
+			nsEmbedCString mimeType;
+			rv = mMIMEInfo->GetMIMEType (mimeType);
+			NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+			helperApp = gnome_vfs_mime_get_default_application (mimeType.get()); 
+
+			nsEmbedString description;
+			mMIMEInfo->GetApplicationDescription (description);
+
+			nsEmbedCString cDesc;
+			NS_UTF16ToCString (description, NS_CSTRING_ENCODING_UTF8, cDesc);
+
+			/* HACK we use the application description to decide
+			   if we have to open the saved file */
+			if ((strcmp (cDesc.get(), "gnome-default") == 0) &&
+			    helperApp)
+#else
 			char *mimeType;
 			rv = mMIMEInfo->GetMIMEType (&mimeType);
 			NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
@@ -360,35 +355,22 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 			PRUnichar *description;
 			mMIMEInfo->GetApplicationDescription (&description);
 
-			/* HACK we use the application description to decide
-			   if we have to open the saved file */
-			if ((strcmp (NS_ConvertUCS2toUTF8 (description).get(), "gnome-default") == 0) &&
-			    helperApp)
-#else
-			nsCAutoString mimeType;
-			rv = mMIMEInfo->GetMIMEType (mimeType);
-			NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
-
-			helperApp = gnome_vfs_mime_get_default_application (mimeType.get()); 
-
-			nsAutoString description;
-			mMIMEInfo->GetApplicationDescription (description);
+			nsEmbedCString cDesc;
+			NS_UTF16ToCString (nsEmbedString(description),
+					   NS_CSTRING_ENCODING_UTF8, cDesc);
 
 			/* HACK we use the application description to decide
 			   if we have to open the saved file */
-			if ((strcmp (NS_ConvertUCS2toUTF8 (description).get(), "gnome-default") == 0) &&
+			if (strcmp (cDesc.get(), "gnome-default") == 0 &&
 			    helperApp)
 #endif
 			{
 				GList *params = NULL;
 				char *param;
-				nsCAutoString aDest;
+				nsEmbedCString aDest;
 
-#if MOZILLA_SNAPSHOT < 16
-				mDestination->GetNativePath (aDest);
-#else
 				mDestination->GetSpec (aDest);
-#endif
+
 				param = gnome_vfs_make_uri_canonical (aDest.get ());
 				params = g_list_append (params, param);
 				gnome_vfs_mime_application_launch (helperApp, params);
@@ -396,10 +378,7 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 
 				g_list_free (params);
 			}
-#if MOZILLA_SNAPSHOT < 16
-			nsMemory::Free (mimeType);
-			nsMemory::Free (description);
-#endif
+
 			gnome_vfs_mime_application_free (helperApp);
 		}
 	}
@@ -531,27 +510,26 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
   
 	PRInt64 timeNow = PR_Now();
   
-	nsAutoString fileDisplayName;
+	nsEmbedString fileDisplayName;
 	inDestFile->GetLeafName(fileDisplayName);
 
-	#if MOZILLA_SNAPSHOT >= 16
+	nsCOMPtr<nsIIOService> ioService;
+	rv = EphyUtils::GetIOService (getter_AddRefs (ioService));
+	NS_ENSURE_SUCCESS (rv, rv);
+
 	nsCOMPtr<nsIURI> destURI;
-	NS_NewFileURI (getter_AddRefs(destURI), inDestFile);
-	#endif
+	ioService->NewFileURI (inDestFile, getter_AddRefs(destURI));
 
 	MozDownload *downloader = new MozDownload ();
 	/* dlListener attaches to its progress dialog here, which gains ownership */
-	#if MOZILLA_SNAPSHOT < 16
-	rv = downloader->InitForEmbed (inOriginalURI, inDestFile, fileDisplayName.get(),
-	#else
 	rv = downloader->InitForEmbed (inOriginalURI, destURI, fileDisplayName.get(),
-	#endif
 				       nsnull, timeNow, webPersist, embedPersist, aMaxSize);
 	NS_ENSURE_SUCCESS (rv, rv);
 
 	PRInt32 flags = nsIWebBrowserPersist::PERSIST_FLAGS_REPLACE_EXISTING_FILES;
 
-	if (!domDocument && !isHTML && !(ephy_flags & EMBED_PERSIST_COPY_PAGE))
+	if (!domDocument && !isHTML && !(ephy_flags & EMBED_PERSIST_COPY_PAGE) &&
+	    !(ephy_flags & EMBED_PERSIST_DO_CONVERSION))
 	{
 		flags |= nsIWebBrowserPersist::PERSIST_FLAGS_NO_CONVERSION;
 	}
@@ -576,22 +554,22 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
 		 * will create the directory as needed.
 		 */
 
-		filesFolder = do_CreateInstance("@mozilla.org/file/local;1");
-		nsAutoString unicodePath;
-		inDestFile->GetPath(unicodePath);
-		filesFolder->InitWithPath(unicodePath);
-      
-		nsAutoString leafName;
-		filesFolder->GetLeafName(leafName);
-		nsAutoString nameMinusExt(leafName);
-		PRInt32 index = nameMinusExt.RFind(".");
-		if (index >= 0)
-		{
-			nameMinusExt.Left(nameMinusExt, index);
-		}
+		nsEmbedCString cPath;
+		inDestFile->GetNativePath (cPath);
 
-		nameMinusExt += NS_LITERAL_STRING (" Files");
-		filesFolder->SetLeafName(nameMinusExt);
+		GString *path = g_string_new (cPath.get());
+		char *dot_pos = strchr (path->str, '.');
+		if (dot_pos)
+		{
+			g_string_truncate (path, dot_pos - path->str);
+		}
+		g_string_append (path, " ");
+		g_string_append (path, _("Files"));
+      
+		filesFolder = do_CreateInstance ("@mozilla.org/file/local;1");
+		filesFolder->InitWithNativePath (nsEmbedCString(path->str));
+
+		g_string_free (path, TRUE);
 
 		rv = webPersist->SaveDocument (domDocument, inDestFile, filesFolder,
 					       contentType, encodingFlags, 80);
@@ -645,6 +623,58 @@ GetFilePath (const char *filename)
 	return path;
 }
 
+static const char*
+file_is_compressed (const char *filename)
+{
+        int i;
+        const char *compression[] = {".gz", ".bz2", ".Z", ".lz", NULL};
+
+        for (i = 0; compression[i] != NULL; i++)
+        {
+                if (g_str_has_suffix (filename, compression[i]))
+                        return compression[i];
+        }
+
+        return NULL;
+}
+
+static const char*
+parse_extension (const char *filename)
+{
+        const char *compression;
+
+        compression = file_is_compressed (filename);
+
+        /* If the file is compressed we might have a double extension */
+        if (compression != NULL)
+        {
+                int i;
+                const char *extensions[] = {"tar", "ps", "xcf", "dvi", "txt", "text", NULL};
+
+                for (i = 0; extensions[i] != NULL; i++)
+                {
+                        char *suffix;
+                        suffix = g_strdup_printf (".%s%s", extensions[i],
+                                                  compression);
+
+                        if (g_str_has_suffix (filename, suffix))
+                        {
+                                char *p;
+
+                                p = g_strrstr (filename, suffix);
+                                g_free (suffix);
+
+                                return p;
+                        }
+
+                        g_free (suffix);
+                }
+        }
+        
+        /* default case */
+        return g_strrstr (filename, ".");
+}
+
 nsresult BuildDownloadPath (const char *defaultFileName, nsILocalFile **_retval)
 {
 	char *path;
@@ -654,11 +684,12 @@ nsresult BuildDownloadPath (const char *defaultFileName, nsILocalFile **_retval)
 	if (g_file_test (path, G_FILE_TEST_EXISTS))
 	{
 		int i = 1;
-		char *dot_pos, *serial = NULL;
+		const char *dot_pos;
+                char *serial = NULL;
 		GString *tmp_path;
 		gssize position;
 
-		dot_pos = g_strrstr (defaultFileName, ".");
+                dot_pos = parse_extension (defaultFileName);
 		if (dot_pos)
 		{
 			position = dot_pos - defaultFileName;
@@ -683,10 +714,9 @@ nsresult BuildDownloadPath (const char *defaultFileName, nsILocalFile **_retval)
 	}
 
 	nsCOMPtr <nsILocalFile> destFile (do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
-	destFile->InitWithNativePath (nsDependentCString (path));
+	destFile->InitWithNativePath (nsEmbedCString (path));
 	g_free (path);
 
 	NS_IF_ADDREF (*_retval = destFile);
 	return NS_OK;
 }
-
