@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.131 2004/08/04 13:52:46 chpe Exp $
+ *  $Id: window-commands.c,v 1.131.4.1 2004/09/22 19:15:17 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -963,7 +963,7 @@ window_cmd_help_about (GtkAction *action,
 	icon_theme = gtk_icon_theme_get_default ();
 	icon_info = gtk_icon_theme_lookup_icon (icon_theme, "web-browser", -1, 0);
 
-	if (icon_info)
+	if (icon_info != NULL)
 	{
 		icon_path = gtk_icon_info_get_filename (icon_info);
 
@@ -971,6 +971,8 @@ window_cmd_help_about (GtkAction *action,
 		{
 			icon = gdk_pixbuf_new_from_file (icon_path, NULL);
 		}
+
+		gtk_icon_info_free (icon_info);
 	}
 	else
 	{
